% Lumped parameter nine degree of freedom plate model
% May 13, 2006
% 2006 Copyright D. E. Adams

clear all;
close all;

% Define parameters and model matrices
k=1e7;
m=0.2;
M=diag([m m m m m m m m m]);
SOk=100;     % boundary condition springs
K=[3*k+k/SOk -k 0  -k -k  0  0  0  0;
   -k 5*k -k -k -k -k  0  0  0;
   0  -k 3*k+k/SOk 0  -k -k  0  0  0;
   -k -k 0  5*k -k  0 -k -k  0;
   -k -k -k -k 8*k -k -k -k -k;
   0  -k -k  0 -k 5*k  0 -k -k;
   0   0  0 -k -k  0 3*k+k/SOk -k  0;
   0   0  0 -k -k -k -k 5*k -k;
   0   0  0  0 -k -k  0 -k 3*k+k/SOk];

% Generate frequency response function
Nf=5000; Ff=5000; % Number of frequency lines and frequency bandwidth
H=zeros(9,9,Nf);
f=linspace(0,Ff,Nf); % Generate frequency vector
a=.001; b=.00001; % Define coefficients for proportional damping model
C=a*M+b*K;  % Assume proportional viscous damping model
for ii = 2:Nf,
    H(:,:,ii)=inv(K-M*(2*pi*f(ii))^2+j*(2*pi*f(ii))*C);
end

% Generate system in MATLAB
A=[zeros(9) eye(9);-inv(M)*K -inv(M)*C];
B=[zeros(1,9) 1 zeros(1,8)]';
sys=ss(A,B,[1 zeros(1,17); zeros(17,18)],zeros(18,1));

% Simulate sinusoidal response to low frequency excitations
Nt=819200;
dt=1e-5;
t=(0:dt:(Nt-1)*dt)';
f1s=100*sin(2*pi*100*t)+1000*sin(2*pi*1000*t);
[x,t]=lsim(sys,f1s,t,zeros(18,1));

% Simulate response to broad band transient excitation
f1n=zeros(Nt,1);
Ntf=2048;
f1n(1:Ntf)=1000*sin(2*pi/t(Ntf)/2*t(1:Ntf));
[xi,t]=lsim(sys,f1n,t,zeros(18,1));

% Perturb plate (either in terms of mass or stiffness)
M=diag([m m m m m m m-0.10*m m m]); % Simulate damage to density (corrosion)
C=a*M+b*K; % Generate damaged proportional viscous damping model
Hd=zeros(9,9,Nf);
for ii = 2:Nf,
    Hd(:,:,ii)=inv(K-M*(2*pi*f(ii))^2+j*(2*pi*f(ii))*C);
end

% Generate system in MATLAB
A=[zeros(9) eye(9);-inv(M)*K -inv(M)*C];
B=[zeros(1,9) 1 zeros(1,8)]';
sys=ss(A,B,[1 zeros(1,17); zeros(17,18)],zeros(18,1));

% Simulate sinusoidal response to low frequency excitations
Nt=819200;
dt=1e-5;
t=(0:dt:(Nt-1)*dt)';
f1s=100*sin(2*pi*100*t)+1000*sin(2*pi*1000*t);
[xd,t]=lsim(sys,f1s,t,zeros(18,1));

% Simulate response to broad band transient excitation
f1n=zeros(Nt,1);
Ntf=2048;
f1n(1:Ntf)=1000*sin(2*pi/t(Ntf)/2*t(1:Ntf));
[xid,t]=lsim(sys,f1n,t,zeros(18,1));

% Plot the baseline and damaged FRFs
figure(1);
semilogy(f,abs(squeeze(H(1,1,:))),'k-',f,abs(squeeze(Hd(1,1,:))),'k:')
xlabel('Frequency [Hz]'); ylabel('Mag H_{11}(\omega) [m/N]');
legend('Baseline','Damaged');
axis([0 5000 1e-10 1e-5]);

% Continue on with noise on data
disp('Press any button to continue');
pause;

% Plot excitations
figure(2);
subplot(211);
plot(t,f1s,'k-');
xlabel('Time [sec]'); 
ylabel('f_1(t) [N]');
axis([0 .05 -1200 1200]);
subplot(212);
plot(t,f1n,'k-');
xlabel('Time [sec]'); 
ylabel('f_1(t) [N]');
axis([0 .05 0 1000]);

% Plot responses
figure(3);
subplot(211);
plot(t,x(:,1),'k-',t,xd(:,1),'k:');
legend('Baseline','Damaged');
xlabel('Time [sec]'); 
ylabel('x_1(t) [m]');
axis([8 8.05 -8e-5 8e-5]);
subplot(212);
plot(t,xi(:,1),'k-',t,xid(:,1),'k:');
legend('Baseline','Damaged');
xlabel('Time [sec]'); 
ylabel('x_1(t) [m]');
axis([0 .5 -6e-5 6e-5]);