% Lumped parameter nine degree of freedom plate model
% selection of transducer positions based on observability
% and controllability concepts
% May 23, 2006
% 2006 Copyright D. E. Adams

clear all;
close all;

% Define parameters and model matrices
k=1e7;
m=0.2;
M=diag([m m m m m m m m m]);
SOk=100;     % boundary condition springs
K=[3*k+k/SOk -k 0  -k -k  0  0  0  0;
   -k 5*k -k -k -k -k  0  0  0;
   0  -k 3*k+k/SOk 0  -k -k  0  0  0;
   -k -k 0  5*k -k  0 -k -k  0;
   -k -k -k -k 8*k -k -k -k -k;
   0  -k -k  0 -k 5*k  0 -k -k;
   0   0  0 -k -k  0 3*k+k/SOk -k  0;
   0   0  0 -k -k -k -k 5*k -k;
   0   0  0  0 -k -k  0 -k 3*k+k/SOk];

% Generate frequency response function
Nf=5000; Ff=5000; % Number of frequency lines and frequency bandwidth
H=zeros(9,9,Nf);
f=linspace(0,Ff,Nf); % Generate frequency vector
a=.001; b=.00001; % Define coefficients for proportional damping model
C=a*M+b*K;  % Assume proportional viscous damping model
C=0*C;
for ii = 2:Nf,
    H(:,:,ii)=inv(K-M*(2*pi*f(ii))^2+j*(2*pi*f(ii))*C);
end

% Generate system in MATLAB
A=[zeros(9) eye(9);-inv(M)*K -inv(M)*C];
B=[zeros(1,9) 1 zeros(1,8)]';
sys=ss(A,B,[1 zeros(1,17); zeros(17,18)],zeros(18,1));

% Compute undamped natural frequencies and indices for frequencies
fn=imag(sqrt(eig(-inv(M)*K))/2/pi);
ind=round(fn/f(2));

% Compute modal deflection shapes
[v,d]=eig(A);
[d,Is]=sort(abs(diag(d)));
v=v(:,Is);
shps=imag([v(1:9,1)/max(abs(v(1:9,1))) v(1:9,3)/max(abs(v(1:9,3))) v(1:9,5)/max(abs(v(1:9,5))) v(1:9,7)/max(abs(v(1:9,7))) v(1:9,9)/max(abs(v(1:9,9))) v(1:9,11)/max(abs(v(1:9,11))) v(1:9,13)/max(abs(v(1:9,13))) v(1:9,15)/max(abs(v(1:9,15))) v(1:9,17)/max(abs(v(1:9,17)))]);

% Plot modal deflection shapes
figure(1);
subplot(911);
plot(shps(:,1),'ko-');
axis([1 9 -1 1]);
grid;
subplot(912);
plot(shps(:,2),'ko-');
grid;
subplot(913);
plot(shps(:,3),'ko-');
grid;
subplot(914);
plot(shps(:,4),'ko-');
grid;
subplot(915);
plot(shps(:,5),'ko-');
grid;
subplot(916);
plot(shps(:,6),'ko-');
grid;
subplot(917);
plot(shps(:,7),'ko-');
grid;
subplot(918);
plot(shps(:,8),'ko-');
grid;
subplot(919);
plot(shps(:,9),'ko-');
grid;
xlabel('Degree of freedom');
ylabel('Modal deflection');

figure(2);
md=5; % Mode number to consider
plot3([0 1 2]',[0 0 0]',shps(1:3,md),'k-o'); hold on;
plot3([0 1 2]',[1 1 1]',shps(4:6,md),'k-o');
plot3([0 1 2]',[2 2 2]',shps(7:9,md),'k-o');
plot3([0 0 0]',[0 1 2]',shps([1 4 7],md),'k-o');
plot3([1 1 1]',[0 1 2]',shps([2 5 8],md),'k-o');
plot3([2 2 2]',[0 1 2]',shps([3 6 9],md),'k-o');
plot3([0 1 2]',[0 0 0]',0*shps(1:3,md),'k:'); hold on;
plot3([0 1 2]',[1 1 1]',0*shps(4:6,md),'k:');
plot3([0 1 2]',[2 2 2]',0*shps(7:9,md),'k:');
plot3([0 0 0]',[0 1 2]',0*shps([1 4 7],md),'k:');
plot3([1 1 1]',[0 1 2]',0*shps([2 5 8],md),'k:');
plot3([2 2 2]',[0 1 2]',0*shps([3 6 9],md),'k:');
axis([0 2 0 2 -3 3]);
xlabel('X');
ylabel('Y');
zlabel('Modal deflection');

% Continue with observability and controllability calculations
disp('Press any key to continue:');
pause;

% Compute observability test matrix
Ct=[1 zeros(1,17)];
N1=[Ct' A'*Ct' (A')^2*Ct' (A')^3*Ct' (A')^4*Ct' (A')^5*Ct' (A')^6*Ct' (A')^7*Ct' (A')^8*Ct' (A')^9*Ct' (A')^10*Ct' (A')^11*Ct' (A')^12*Ct' (A')^13*Ct' (A')^14*Ct' (A')^15*Ct' (A')^16*Ct' (A')^17*Ct'];
Ct=[0 1 zeros(1,16)];
N2=[Ct' A'*Ct' (A')^2*Ct' (A')^3*Ct' (A')^4*Ct' (A')^5*Ct' (A')^6*Ct' (A')^7*Ct' (A')^8*Ct' (A')^9*Ct' (A')^10*Ct' (A')^11*Ct' (A')^12*Ct' (A')^13*Ct' (A')^14*Ct' (A')^15*Ct' (A')^16*Ct' (A')^17*Ct'];
Ct=[0 0 1 zeros(1,15)];
N3=[Ct' A'*Ct' (A')^2*Ct' (A')^3*Ct' (A')^4*Ct' (A')^5*Ct' (A')^6*Ct' (A')^7*Ct' (A')^8*Ct' (A')^9*Ct' (A')^10*Ct' (A')^11*Ct' (A')^12*Ct' (A')^13*Ct' (A')^14*Ct' (A')^15*Ct' (A')^16*Ct' (A')^17*Ct'];
Ct=[0 0 0 1 zeros(1,14)];
N4=[Ct' A'*Ct' (A')^2*Ct' (A')^3*Ct' (A')^4*Ct' (A')^5*Ct' (A')^6*Ct' (A')^7*Ct' (A')^8*Ct' (A')^9*Ct' (A')^10*Ct' (A')^11*Ct' (A')^12*Ct' (A')^13*Ct' (A')^14*Ct' (A')^15*Ct' (A')^16*Ct' (A')^17*Ct'];
Ct=[0 0 0 0 1 zeros(1,13)];
N5=[Ct' A'*Ct' (A')^2*Ct' (A')^3*Ct' (A')^4*Ct' (A')^5*Ct' (A')^6*Ct' (A')^7*Ct' (A')^8*Ct' (A')^9*Ct' (A')^10*Ct' (A')^11*Ct' (A')^12*Ct' (A')^13*Ct' (A')^14*Ct' (A')^15*Ct' (A')^16*Ct' (A')^17*Ct'];
Ct=eye(18);
N9=[Ct' A'*Ct' (A')^2*Ct' (A')^3*Ct' (A')^4*Ct' (A')^5*Ct' (A')^6*Ct' (A')^7*Ct' (A')^8*Ct' (A')^9*Ct' (A')^10*Ct' (A')^11*Ct' (A')^12*Ct' (A')^13*Ct' (A')^14*Ct' (A')^15*Ct' (A')^16*Ct' (A')^17*Ct'];

figure(3);
semilogy(1:18,svd(N1)/norm(N1),'k.-',1:18,svd(N2)/norm(N2),'k-o',1:18,svd(N3)/norm(N3),'k-x',1:18,svd(N4)/norm(N4),'k-+',1:18,svd(N5)/norm(N5),'k-s',1:18,svd(N9)/norm(N9),'k-d')
axis([1 18 1e-90 1e0])
xlabel('Number');
ylabel('Normalized Singular Value');