% Six DOF panel system model
% Pressure and temperature simulation
% 4/14/06 
% 2006 Copyright D. E. Adams

% Define parameters
load panelmatrices
a=2*0.1104;  % Define length of half diamond
b=a;
Ar=a*a;
K1=P(1); K2=P(2); K3=P(3); K4=P(4); K5=P(5); K6=P(6); K7=P(7);
Kt1=P(8); Kt2=P(9); Kt3=P(10);
K=[K2+K6+K7 -K6       -K7             0            0                             0
   -K6      K4+K6+K7  0              -K7           0                             0
   -K7      0         K1+K5+K7       -K5           -a*K1/2+a*K5/2                b*K5/2
   0        -K7       -K5            K3+K5+K7      -a*K5/2                       b*K3/2-b*K5/2
   0        0         -a*K1/2+a*K5/2 -K5*a/2       Kt1+Kt2+K1*(a/2)^2+K5*(a/2)^2 -Kt2+K5*(a/2)*(b/2)
   0        0         b*K5/2         b*K3/2-b*K5/2 -Kt2+K5*(a/2)*(b/2)           Kt3+Kt2+K3*(b/2)^2+K5*(b/2)^2];

% Setup time vector
Npts=5e5;
t=linspace(0,100,Npts);
dt=t(2)-t(1);

% Simulate multiple flights of panel

% Specify number of pressure and temp levels
N=25;
Nsh=5e4;
tm=t(1:Nsh);

% Define pressure and temperature sequences
Pset0=10.^([1.8 1.6 1.4 1.0 0.6 0.4 0.2 0.1 0.1 0.1 0.2 0.6 0.8 1.0 1.2 1.3 1.5 1.5 1.4 1.2 0.8 0.7 0.3 0.1 0.1]*100/20);
Tset0=[80 100 200 220 400 500 600 820 1100 1300 1400 1400 1300 1300 1200 1000 900 620 580 500 420 400 400 400 80];

% Define temperature distribution for standoffs on backside of panel
gamma=0.3;
Tpset0=(Tset0-80)*gamma+ones(1,25)*80;

% Specify variation of moduli with temperature 
dT=Tset0-70;   % Temperature of panel minus room temperature
dTp=Tpset0-70; % Temperature of standoff minus room temperature
Ema754=(21.6e6)*(0.9988-(1.306e-4)*(dT)-(4.298e-8)*(dT).^2);
E617=(30.6e6)*(0.9988-(1.306e-4)*(dT)-(4.298e-8)*(dT).^2);

% Simulate one flight
kk=1;

% Generate force and temperature variations (with normal distributions)
Var=30;
r1=randn(1,N)/Var;
r2=randn(1,N)/Var;
Pset=Pset0+r1.*Pset0; Prun(kk,:)=Pset;
Tset=Tset0+r2.*Tset0; Trun(kk,:)=Tset;
Tpset=Tpset0+r2.*Tpset0;
x0=zeros(2*max(size(MP)),1);

% Simulate flight
for ii = 1:N,
    
F1=Pset(ii)*randn(Nsh,1)*Ar*4.44/(2.54e-2)^2;
F2=Pset(ii)*randn(Nsh,1)*Ar*4.44/(2.54e-2)^2;

dTN=Tset(ii)-70;   % Temperature of panel minus room temperature
dTpN=Tpset(ii)-70; % Temperature of standoffs minus room temperature
Ema754N=(0.9988-(1.306e-4)*(dTpN)-(4.298e-8)*(dTpN).^2);
E617N=(0.9988-(1.306e-4)*(dTN)-(4.298e-8)*(dTN).^2);

% Standoffs and bolt rotational stiffnesses
K1m=K1*Ema754N;
K2m=K2*Ema754N;
K3m=K3*Ema754N;
K4m=K4*Ema754N;
K7m=K7*Ema754N;
Kt1m=Kt1*Ema754N;
Kt3m=Kt3*Ema754N;

% Panel stiffnesses
K5m=K5*E617N;
K6m=K6*E617N;
Kt2m=Kt2*E617N;

Km(:,ii)=[K1m K2m K3m K4m K7m Kt1m Kt2m K3m K5m K6m Kt3m]';

% Generate new stiffness and damping matrices
K=[K2m+K6m+K7m -K6m        -K7m              0              0                                 0
   -K6m        K4m+K6m+K7m 0                -K7m            0                                 0
   -K7m        0           K1m+K5m+K7m      -K5m            -a*K1m/2+a*K5m/2                  b*K5m/2
   0           -K7m        -K5m             K3m+K5m+K7m     -a*K5m/2                          b*K3m/2-b*K5m/2
   0           0           -a*K1m/2+a*K5m/2 -K5m*a/2        Kt1m+Kt2m+K1m*(a/2)^2+K5m*(a/2)^2 -Kt2m+K5m*(a/2)*(b/2)
   0           0           b*K5m/2          b*K3m/2-b*K5m/2 -Kt2m+K5m*(a/2)*(b/2)             Kt3m+Kt2m+K3m*(b/2)^2+K5m*(b/2)^2];
alpha=.00006; beta=.00006;
C=alpha*MP+beta*K;

% Define state space system matrices
sys=ss([zeros(6) eye(6); -inv(MP)*K -inv(MP)*C],eye(12),eye(12),zeros(12,12));

% Simulate the (piecewise) linear response
[y,T,x]=lsim(sys,[zeros(Nsh,1) zeros(Nsh,1) zeros(Nsh,1) zeros(Nsh,1) zeros(Nsh,1) zeros(Nsh,1) 1*F1/4 1*F1/4 F2 F2 zeros(Nsh,1) zeros(Nsh,1)],tm,x0);

% Reset initial conditions for next segment
x0=x(Nsh,:);

% Calculate transmissibility
Y1=x(:,3)-a/2*x(:,5);
Y2=x(:,4)+b/2*x(:,6);
X1=x(:,1);
X2=x(:,2);

rts(:,ii)=sort(imag(sqrt(eig(-inv(MP)*K))));

end % loop back to next mission segment

% Plot results
figure(1);
subplot(211);
plot(T,Y1,'k-',T,Y2,'k--');
xlabel('Time [sec]');
ylabel('Response y_1 and y_2 [m]');
axis([0 0.2 -5e-3 5e-3]);
subplot(212);
plot(Tset0,rts,'k.-');
xlabel('Temperature [deg F]');
ylabel('Natural Frequencies [rad/s]');

pause;
disp('Press any key to continue');

% Animate motion of panel
figure(2);
for ii = 1:250,
    plot([-22e-2+x(ii,1) -1e-2+x(ii,1)],[x(ii,3)-a/2*x(ii,5) x(ii,3)+a/2*x(ii,5)],'k.-','LineWidth',10,'MarkerSize',24); hold on;
    plot([1e-2+x(ii,2) 22e-2+x(ii,2)],[x(ii,4)-b/2*x(ii,6) x(ii,4)+b/2*x(ii,6)],'k.-','LineWidth',10,'MarkerSize',24); hold off;
    axis([[-30e-2 30e-2 -4e-3 4e-3]]);
    pause(.01);
end