% One dimensional rod
% Longitudinal wave propagation
% 4/12/06 
% 2006 Copyright D. E. Adams

clear all;
close all;

% Parameters
Ne=17;
A=(2.54e-2)^2;
E=70e9;
L=0.1;     % Length of component
Le=L/Ne;   % Length of each element  
V=A*L;
rho=2700*V/L;  % Density per unit length
Ks=0.01*A*E/Le;

% Generate matrices
m=[rho*Le/3 rho*Le/6;rho*Le/6 rho*Le/3];
k=[A*E/Le -A*E/Le; -A*E/Le A*E/Le];

% Generate total mass and stiffness matrices
M=zeros(Ne+1);
K=zeros(Ne+1);
for ii = 1:Ne,
    M(ii:ii+1,ii:ii+1)=M(ii:ii+1,ii:ii+1)+m;
    K(ii:ii+1,ii:ii+1)=K(ii:ii+1,ii:ii+1)+k;
end

% Apply the boundary conditions
K(1,1)=K(1,1)+Ks;
K(Ne+1,Ne+1)=K(Ne+1,Ne+1)+Ks;

% Generate damping matrix and state space matrices
alpha=0e-15;
beta=0e-15;
C=alpha*M+beta*K;
A=[zeros(Ne+1) eye(Ne+1);-inv(M)*K -inv(M)*C];
B=[zeros(1,Ne+1) 1 zeros(1,Ne)]';

% Generate system in MATLAB
sys=ss(A,B,[eye(Ne+1) zeros(Ne+1)],zeros(Ne+1,1));

% Simulate response
Nt=1024;
dt=2e-7;
xmax=.00004;
ymax=5e-7;
t=(0:dt:(Nt-1)*dt)';
f1=zeros(Nt,1);
f1(1:100)=1e6*hanning(100).*sin(linspace(0,6*pi,100))';
[x,t]=lsim(sys,f1,t);

% Plot force
plot((0:1/t(Nt):(Nt-1)/t(Nt)),abs(fft(f1)/Nt/2),'k-');
ylabel('F_{1}(\omega) [N]')
xlabel('Frequency [Hz]');
xlim([0 500e3]);

% Plot response
figure(2);
subplot(611);
plot(t,f1(:,1),'k');
ylabel('f_{1} [N]')
xlim([0 xmax]);
subplot(612);
plot(t,x(:,1),'k');
ylabel('x_{1} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);
subplot(613);
plot(t,x(:,4),'k');
ylabel('x_{4} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);
subplot(614);
plot(t,x(:,8),'k');
ylabel('x_{8} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);
subplot(615);
plot(t,x(:,12),'k');
ylabel('x_{12} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);
subplot(616);
plot(t,x(:,16),'k');
xlabel('Time [sec]');
ylabel('x_{16} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);

% Insert the crack
crackind=8;
K(crackind:crackind+1,crackind:crackind+1)=K(crackind:crackind+1,crackind:crackind+1)-0.1*k;

% Generate damping matrix and state space matrices
C=alpha*M+beta*K;
A=[zeros(Ne+1) eye(Ne+1);-inv(M)*K -inv(M)*C];
B=[zeros(1,Ne+1) 1 zeros(1,Ne)]';

% Generate system in MATLAB
sys=ss(A,B,[eye(Ne+1) zeros(Ne+1)],zeros(Ne+1,1));

% Simulate response
[xd,t]=lsim(sys,f1,t);

% Compute difference in response with and without damage
x=xd-x;

% Plot difference bewteen responses with and without damage
ymax=ymax/10;
figure(3);
subplot(511);
plot(t,x(:,1),'k');
ylabel('\Delta x_{1} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);
subplot(512);
plot(t,x(:,4),'k');
ylabel('\Delta x_{4} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);
subplot(513);
plot(t,x(:,8),'k');
ylabel('\Delta x_{8} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);
subplot(514);
plot(t,x(:,12),'k');
ylabel('\Delta x_{12} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);
subplot(515);
plot(t,x(:,16),'k');
xlabel('Time [sec]');
ylabel('\Delta x_{16} [m]');
xlim([0 xmax]);
ylim([-ymax ymax]);